package models

import (
	"strings"

	"github.com/juju/errors"
)

// APITypeName
type APITypeName string

const (
	APITypeName_GIN_GONIC   APITypeName = "Gin Gonic"
	APITypeName_GO_SWAGGER  APITypeName = "Go Swagger"
	APITypeName_MOJOLICIOUS APITypeName = "Mojolicious"
	APITypeName_NULL        APITypeName = ""
)

// GetListOfAPITypeName returns a list of APITypeName
func GetListOfAPITypeName() []APITypeName {
	return []APITypeName{
		APITypeName_GIN_GONIC,
		APITypeName_GO_SWAGGER,
		APITypeName_MOJOLICIOUS,
	}
}

// IsValid validates enum values
func (e APITypeName) IsValid() bool {
	for _, v := range GetListOfAPITypeName() {
		if e == v {
			return true
		}
	}

	return false
}

func NewAPITypeName(in string) (APITypeName, error) {
	out := APITypeName_NULL
	if in != "" {
		out = APITypeName(in)
		if !out.IsValid() {
			return APITypeName_NULL, errors.BadRequestf("Value %s invalid for enum APITypeName", in)
		}
	}
	return out, nil
}

// NewAPITypeNameFromInput
func NewAPITypeNameFromInput(in string) (APITypeName, error) {

	in = strings.ToLower(in)
	in = strings.ReplaceAll(in, " ", "-")
	in = strings.ReplaceAll(in, "_", "-")

	out := APITypeName_NULL
	if in != "" {
		for _, apiTypeName := range GetListOfAPITypeName() {
			apiTypeStr := strings.ToLower(string(apiTypeName))
			apiTypeStr = strings.ReplaceAll(apiTypeStr, " ", "-")
			apiTypeStr = strings.ReplaceAll(apiTypeStr, "_", "-")
			if in == apiTypeStr {
				return apiTypeName, nil
			}
		}
		return APITypeName_NULL, errors.BadRequestf("Value %s invalid for enum APITypeName", in)
	}
	return out, nil
}
